# (c) Copyright 2014,2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.conf import settings
from horizon_web_ui.freezer_ui.api import api
from mock import patch
from openstack_auth import utils
import openstack_dashboard.test.helpers as helpers


@patch('freezer.apiclient.client')
class TestApi(helpers.TestCase):
    CONFIG = {u'user_name': u'admin',
              u'config_id': u'053a62e0-66a9-4a1c-ba58-6b7348d22166',
              u'config_file': {u'start_datetime': u'1432736797',
                               u'repeat': u'1440',
                               u'max_priority': False,
                               u'encryption_password': u'secret',
                               u'src_file': u'fdsfsdds',
                               u'clients': [u'test-client'],
                               u'levels': 0,
                               u'proxy': u'',
                               u'container_name': u'dummy_container',
                               u'exclude': u'/tmp',
                               u'compression': u'gzip',
                               u'log_file': u'',
                               u'optimize': u'speed',
                               u'name': u'fdsfs'},
              u'user_id': u'13c2b15308c04cdf86989ee7335eb504'}

    def setUp(self):
        super(TestApi, self).setUp()

        # Usually this monkey patching happens in urls.py. This doesn't work
        # here because we never invoke urls.py in this test. So we have to do
        # it manually.
        utils.patch_middleware_get_user()

    def _setup_request(self):
        super(TestApi, self)._setup_request()
        # For some strange reason, Horizon sets the token to the token id
        # rather than the token object. This fixes it.
        self.request.session['token'] = self.token

    def assert_client_got_created(self, client_mock):
        client_mock.Client.assert_called_with(
            token=self.request.session['token'].id,
            auth_url=settings.OPENSTACK_KEYSTONE_URL,
            endpoint=settings.FREEZER_API_URL)

    def test_configuration_delete(self, client_mock):
        api.configuration_delete(
            self.request, u'053a62e0-66a9-4a1c-ba58-6b7348d22166')

        self.assert_client_got_created(client_mock)
        client_mock.Client().configs.delete.\
            assert_called_once_with(u'053a62e0-66a9-4a1c-ba58-6b7348d22166')

    def test_configuration_clone(self, client_mock):
        client_mock.Client().configs.get.return_value = [self.CONFIG]
        client_mock.Client().configs.\
            create.return_value = u'28124cf0-6cd3-4b38-a0e9-b6f41568fa37'

        result = api.configuration_clone(
            self.request, u'053a62e0-66a9-4a1c-ba58-6b7348d22166')

        self.assertEqual(result, u'28124cf0-6cd3-4b38-a0e9-b6f41568fa37')
        self.assert_client_got_created(client_mock)
        data = self.CONFIG[u'config_file']
        data['name'] = 'fdsfs_clone'
        client_mock.Client().configs.create.assert_called_once_with(data)

    def test_configuration_get(self, client_mock):
        client_mock.Client().configs.get.return_value = [self.CONFIG]

        result = api.configuration_get(
            self.request, u'053a62e0-66a9-4a1c-ba58-6b7348d22166')

        self.assertEqual(1, len(result))
        # Test if properties are accessible via object properties
        self.assertEqual(u'admin', result[0].user_name)
        # Test if nested properties are accessible via object properties
        self.assertEqual(u'1432736797', result[0].start_datetime)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

import datetime
import logging
import pprint

from django.template.defaultfilters import date as django_date
from django.utils.translation import ugettext_lazy as _
from django.views import generic

from horizon import tables
from horizon import workflows

import disaster_recovery.api.api as freezer_api

from disaster_recovery.backups import tables as freezer_tables
from disaster_recovery.backups.workflows import restore as restore_workflow
from disaster_recovery.utils import shield


LOG = logging.getLogger(__name__)


class IndexView(tables.DataTableView):
    name = _("Backups")
    slug = "backups"
    table_class = freezer_tables.BackupsTable
    template_name = "disaster_recovery/backups/index.html"

    @shield('Unable to retrieve backups.', redirect='backups:index')
    def get_data(self):
        filters = self.table.get_filter_string() or None
        return freezer_api.Backup(self.request).list(search=filters)


class DetailView(generic.TemplateView):
    template_name = 'disaster_recovery/backups/detail.html'

    @shield('Unable to get backup.', redirect='backups:index')
    def get_context_data(self, **kwargs):
        backup = freezer_api.Backup(self.request).get(kwargs['backup_id'],
                                                      json=True)
        return {'data': pprint.pformat(backup)}


class RestoreView(workflows.WorkflowView):
    workflow_class = restore_workflow.Restore

    @shield('Unable to get backup.', redirect='backups:index')
    def get_object(self, *args, **kwargs):
        return freezer_api.Backup(self.request).get(self.kwargs['backup_id'])

    def is_update(self):
        return 'name' in self.kwargs and bool(self.kwargs['name'])

    @shield('Unable to get backup.', redirect='backups:index')
    def get_workflow_name(self):
        backup = freezer_api.Backup(self.request).get(self.kwargs['backup_id'])
        backup_date = datetime.datetime.fromtimestamp(int(backup.time_stamp))
        backup_date_str = django_date(backup_date,
                                      'SHORT_DATETIME_FORMAT')
        return "Restore '{}' from {}".format(backup.backup_name,
                                             backup_date_str)

    def get_initial(self):
        return {"backup_id": self.kwargs['backup_id']}

    @shield('Unable to get backup.', redirect='backups:index')
    def get_workflow(self, *args, **kwargs):
        workflow = super(RestoreView, self).get_workflow(*args, **kwargs)
        workflow.name = self.get_workflow_name()
        return workflow



# (c) Copyright 2014,2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import logging

from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ungettext_lazy

from horizon import tables
from horizon.utils.urlresolvers import reverse

import disaster_recovery.api.api as freezer_api


LOG = logging.getLogger(__name__)


class ObjectFilterAction(tables.FilterAction):
    def allowed(self, request, datum):
        return bool(self.table.kwargs['session_id'])


def get_link(session):
    return reverse('horizon:disaster_recovery:sessions:index',
                   kwargs={'session_id': session.session_id})


class CreateJob(tables.LinkAction):
    name = "create_session"
    verbose_name = _("Create Session")
    url = "horizon:disaster_recovery:sessions:create"
    classes = ("ajax-modal",)
    icon = "plus"


class DeleteSession(tables.DeleteAction):
    name = "delete"
    classes = ("btn-danger",)
    icon = "remove"
    help_text = _("Delete sessions is not recoverable.")

    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Delete Session",
            u"Delete Sessions",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Deleted Session",
            u"Deleted Sessions",
            count
        )

    def delete(self, request, session_id):
        return freezer_api.Session(request).delete(session_id)


class EditSession(tables.LinkAction):
    name = "edit_session"
    verbose_name = _("Edit Session")
    classes = ("ajax-modal",)
    icon = "pencil"

    def get_link_url(self, datum=None):
        return reverse("horizon:disaster_recovery:sessions:edit",
                       kwargs={'session_id': datum.session_id})


class DeleteMultipleActions(DeleteSession):
    name = "delete_multiple_actions"


class DeleteJobFromSession(tables.DeleteAction):
    name = "delete_job_from_session"
    classes = ("btn-danger",)
    icon = "remove"
    help_text = _("Delete jobs is not recoverable.")

    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Delete Job",
            u"Delete Jobs",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Deleted Job",
            u"Deleted Jobs",
            count
        )

    def delete(self, request, session):
        job_id, session_id = session.split('===')
        return freezer_api.Session(request).remove_job(session_id, job_id)


class JobsTable(tables.DataTable):
    client_id = tables.Column(
        'client_id',
        verbose_name=_("Client ID"))

    result = tables.Column(
        'result',
        verbose_name=_("Status"))

    def get_object_id(self, job):
        # this is used to pass to values as an url
        # TODO(m3m0): look for a way to improve this
        ids = '{0}==={1}'.format(job.job_id, job.session_id)
        return ids

    class Meta(object):
        name = "jobs"
        verbose_name = _("Jobs")
        table_actions = (ObjectFilterAction,)
        row_actions = (DeleteJobFromSession,)
        footer = False
        multi_select = True


class SessionsTable(tables.DataTable):
    description = tables.Column('description',
                                link=get_link,
                                verbose_name=_("Session"))

    status = tables.Column('status',
                           verbose_name=_("Status"))

    class Meta(object):
        name = "sessions"
        verbose_name = _("Sessions")
        table_actions = (ObjectFilterAction,
                         CreateJob,
                         DeleteMultipleActions)
        row_actions = (EditSession,
                       DeleteSession,)
        footer = False
        multi_select = True



# (c) Copyright 2014,2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

from django.utils.translation import ugettext_lazy as _
from horizon.utils.urlresolvers import reverse

from horizon import exceptions
from horizon import forms
from horizon import workflows

import disaster_recovery.api.api as freezer_api


LOG = logging.getLogger(__name__)


class SessionConfigurationAction(workflows.Action):
    session_id = forms.ChoiceField(
        help_text=_("Set a session to attach this job"),
        label=_("Session Name"),
        required=True)

    job_id = forms.CharField(
        widget=forms.HiddenInput(),
        required=True)

    def populate_session_id_choices(self, request, context):
        sessions = []
        try:
            sessions = freezer_api.Session(request).list()
        except Exception:
            exceptions.handle(request, _('Error getting session list'))

        sessions = [(s.session_id, s.description) for s in sessions]
        sessions.insert(0, ('', _('Select A Session')))
        return sessions

    class Meta:
        name = _("Sessions")
        slug = "sessions"


class SessionConfiguration(workflows.Step):
    action_class = SessionConfigurationAction
    contributes = ('session_id',
                   'job_id')


class AttachJobToSession(workflows.Workflow):
    slug = "attach_job"
    name = _("Attach To Session")
    finalize_button_name = _("Attach")
    success_message = _('Job saved successfully.')
    failure_message = _('Unable to attach to session.')
    success_url = "horizon:disaster_recovery:jobs:index"
    default_steps = (SessionConfiguration,)

    def handle(self, request, context):
        try:
            freezer_api.Session(request).add_job(context['session_id'],
                                                 context['job_id'])

            return reverse("horizon:disaster_recovery:jobs:index")
        except Exception:
            exceptions.handle(request)
            return False



# (c) Copyright 2014,2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
URL patterns for the OpenStack Dashboard.
"""

from django.conf.urls import patterns
from django.conf.urls import url

import rest_api

urlpatterns = patterns(
    '',
    url(r'^api/clients/$', rest_api.Clients.as_view(), name="api_clients"),
    url(r'^api/actions/$', rest_api.ActionList.as_view(), name="api_actions"),
    url(r'^api/actions/job/(?P<job_id>[^/]+)?$',
        rest_api.Actions.as_view(), name="api_actions_in_job"),
)



# (c) Copyright 2014,2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

import datetime

from django import shortcuts
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import workflows

import disaster_recovery.api.api as freezer_api


LOG = logging.getLogger(__name__)


class InfoConfigurationAction(workflows.Action):

    description = forms.CharField(
        label=_("Job Name"),
        help_text=_("Set a name for this job"),
        required=True)

    job_id = forms.CharField(
        widget=forms.HiddenInput(),
        required=False)

    schedule_start_date = forms.CharField(
        label=_("Start Date and Time"),
        required=False)

    schedule_interval = forms.CharField(
        label=_("Interval"),
        required=False,
        help_text=_("Repeat this configuration in a minutes interval."))

    schedule_end_date = forms.CharField(
        label=_("End Date and Time"),
        required=False)

    def __init__(self, request, context, *args, **kwargs):
        self.request = request
        self.context = context
        super(InfoConfigurationAction, self).__init__(
            request, context, *args, **kwargs)

    def clean(self):
        cleaned_data = super(InfoConfigurationAction, self).clean()
        self._check_start_datetime(cleaned_data)
        self._check_end_datetime(cleaned_data)
        return cleaned_data

    def _validate_iso_format(self, start_date):
        try:
            return datetime.datetime.strptime(
                start_date, "%Y-%m-%dT%H:%M:%S")
        except ValueError:
            return False

    def _check_start_datetime(self, cleaned_data):
        if cleaned_data.get('start_datetime') and not \
                self._validate_iso_format(
                    cleaned_data.get('schedule_start_date')):
            msg = _("Start date time is not in ISO format.")
            self._errors['schedule_start_date'] = self.error_class([msg])

    def _check_end_datetime(self, cleaned_data):
        if cleaned_data.get('end_datetime') and not \
                self._validate_iso_format(
                    cleaned_data.get('schedule_end_date')):
            msg = _("End date time is not in ISO format.")
            self._errors['schedule_end_date'] = self.error_class([msg])

    class Meta(object):
        name = _("Job Info")
        slug = "info"
        help_text_template = "disaster_recovery/jobs" \
                             "/_scheduling.html"


class InfoConfiguration(workflows.Step):
    action_class = InfoConfigurationAction
    contributes = ('description',
                   'job_id',
                   'actions',
                   'schedule_start_date',
                   'schedule_interval',
                   'schedule_end_date')


class UpdateJob(workflows.Workflow):
    slug = "update_job"
    name = _("Update Job")
    finalize_button_name = _("Save")
    success_message = _('Job created correctly.')
    failure_message = _('Unable to created job.')
    success_url = "horizon:disaster_recovery:jobs:index"
    default_steps = (InfoConfiguration,)

    def handle(self, request, context):
        try:
            if context['job_id'] != '':
                freezer_api.Job(request).update(context['job_id'], context)
            return shortcuts.redirect('horizon:disaster_recovery:jobs:index')
        except Exception:
            exceptions.handle(request)
            return False



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

# Some helper functions to use the disaster_recovery client functionality
# from horizon.

import logging

from django.conf import settings

from horizon.utils.memoized import memoized  # noqa

import freezer.apiclient.client
from disaster_recovery import utils


LOG = logging.getLogger(__name__)


@memoized
def client(request):
    """Return a freezer client object"""
    api_url = _get_service_url(request)

    # get keystone version to connect to
    ks_version = getattr(settings,
                         'OPENSTACK_API_VERSIONS', {}).get('identity', 2.0)

    credentials = {
        'token': request.user.token.id,
        'auth_url': getattr(settings, 'OPENSTACK_KEYSTONE_URL'),
        'endpoint': api_url,
        'version': ks_version
    }

    if ks_version == 3:
        credentials['project_name'] = request.user.project_name
        credentials['project_domain_name'] = \
            request.user.domain_name or 'Default'

    return freezer.apiclient.client.Client(**credentials)


@memoized
def _get_service_url(request):
    """Get freezer api url"""
    catalog = (getattr(request.user, "service_catalog", None))
    if not catalog:
        return _get_hardcoded_url()

    for c in catalog:
        if c['name'] == 'freezer':
            for e in c['endpoints']:
                return e['internalURL']
    else:
        return _get_hardcoded_url()


@memoized
def _get_hardcoded_url():
    """In case freezer is not registered in keystone catalog, look for it in
    local_settings.py
    :return: freezer_api_url
    """
    try:
        LOG.warn('Using hardcoded FREEZER_API_URL at {0}'
                 .format(settings.FREEZER_API_URL))
        return getattr(settings, 'FREEZER_API_URL', None)
    except Exception:
        LOG.warn('No FREEZER_API_URL was found in local_settings.py')
        raise


class Job(object):

    def __init__(self, request):
        self.request = request
        self.client = client(request)

    def list(self, json=False, limit=100, offset=0, search=None):
        if search:
            search = {"match": [{"_all": search}, ], }

        jobs = self.client.jobs.list_all(limit=limit,
                                         offset=offset,
                                         search=search)
        if json:
            return jobs

        return [utils.JobObject(
            job.get('job_id'),
            job.get('description'),
            job.get('job_schedule', {}).get('result'),
            job.get('job_schedule', {}).get('event'),
            job.get('client_id')
        ) for job in jobs]

    def get(self, job_id, json=False):
        job = self.client.jobs.get(job_id)

        if json:
            return job

        return utils.JobObject(
            job.get('job_id'),
            job.get('description'),
            job.get('job_schedule', {}).get('result'),
            job.get('job_schedule', {}).get('event'),
            job.get('client_id'))

    def create(self, job):
        return self._build(job)

    def update(self, job_id, job):
        scheduling = {}
        try:
            if job['schedule_end_date'] != '':
                utils.assign_and_remove(job, scheduling, 'schedule_end_date')
            else:
                job.pop('schedule_end_date')
        except KeyError:
            pass

        try:
            if job['schedule_interval'] != '':
                utils.assign_and_remove(job, scheduling, 'schedule_interval')
            else:
                job.pop('schedule_interval')
        except KeyError:
            pass

        try:
            if job['schedule_start_date'] != '':
                utils.assign_and_remove(job, scheduling, 'schedule_start_date')
            else:
                job.pop('schedule_start_date')
        except KeyError:
            pass

        job.pop('job_actions', [])
        job.pop('clients', None)
        job.pop('actions', None)
        job.pop('job_id')
        job['job_schedule'] = scheduling
        return self.client.jobs.update(job_id, job)

    def update_actions(self, job_id, action_ids):
        ids = utils.get_action_ids(action_ids)
        job = self.get(job_id, json=True)
        job.pop('job_actions', None)
        actions = self._get_actions_in_job(ids)
        job['job_actions'] = actions
        return self.client.jobs.update(job_id, job)

    def delete(self, job_id):
        return self.client.jobs.delete(job_id)

    def actions(self, job_id, api=False):
        job = self.get(job_id, json=True)

        if not job:
            return []

        if api:
            return job.get('job_actions', [])

        return [utils.ActionObject(
            action_id=a.get('action_id'),
            action=a.get('freezer_action', {}).get('action'),
            backup_name=a.get('freezer_action', {}).get('backup_name'),
            job_id=job_id
        ) for a in job.get('job_actions')]

    def delete_action(self, ids):
        action_id, job_id = ids.split('===')
        job = self.get(job_id, json=True)
        for action in job['job_actions']:
            if action.get('action_id') == action_id:
                job.get('job_actions').remove(action)
        return self.client.jobs.update(job_id, job)

    def clone(self, job_id):
        job = self.get(job_id, json=True)
        job['description'] = '{0}_clone'.format(job['description'])
        job.pop('job_id', None)
        job.pop('_version', None)
        job_id = self.client.jobs.create(job)
        return self.stop(job_id)

    def stop(self, job_id):
        return self.client.jobs.stop_job(job_id)

    def start(self, job_id):
        return self.client.jobs.start_job(job_id)

    def _build(self, job):
        action_ids = utils.get_action_ids(job.pop('actions'))
        job = utils.create_dict(**job)
        clients = job.pop('clients', [])
        scheduling = {}
        new_job = {}
        utils.assign_and_remove(job, scheduling, 'schedule_start_date')
        utils.assign_and_remove(job, scheduling, 'schedule_interval')
        utils.assign_and_remove(job, scheduling, 'schedule_end_date')

        actions = self._get_actions_in_job(action_ids)

        new_job['description'] = job.get('description')
        new_job['job_actions'] = actions
        new_job['job_schedule'] = scheduling

        for client_id in clients:

            search = client_id
            client = Client(self.request).list(search=search)

            new_job['client_id'] = client[0].id
            job_id = self.client.jobs.create(new_job)
            self.stop(job_id)
        return True

    def _get_actions_in_job(self, action_ids):
        actions_in_job = []
        for action_id in action_ids:
            action = Action(self.request).get(action_id, json=True)
            a = {
                'action_id': action['action_id'],
                'freezer_action': action['freezer_action']
            }
            actions_in_job.append(a)
        return actions_in_job


class Session(object):

    def __init__(self, request):
        self.request = request
        self.client = client(request)

    def list(self, json=False, limit=30, offset=0, search=None):
        if search:
            search = {"match": [{"_all": search}, ], }

        sessions = self.client.sessions.list_all(limit=limit,
                                                 offset=offset,
                                                 search=search)

        if json:
            return sessions

        return [utils.SessionObject(
            session.get('session_id'),
            session.get('description'),
            session.get('status'),
            session.get('jobs'),
            session.get('schedule', {}).get('schedule_start_date'),
            session.get('schedule', {}).get('schedule_interval'),
            session.get('schedule', {}).get('schedule_end_date')
        ) for session in sessions]

    def get(self, session_id, json=False):
        session = self.client.sessions.get(session_id)

        if json:
            return session

        return utils.SessionObject(
            session.get('session_id'),
            session.get('description'),
            session.get('status'),
            session.get('jobs'),
            session.get('schedule', {}).get('schedule_start_date'),
            session.get('schedule', {}).get('schedule_interval'),
            session.get('schedule', {}).get('schedule_end_date'))

    def create(self, session):
        return self._build(session)

    def update(self, session, session_id):
        return self.client.sessions.update(session_id, session)

    def delete(self, session_id):
        return self.client.sessions.delete(session_id)

    def remove_job(self, session_id, job_id):
        try:
            # even if the job is removed from the session the api returns an
            # error.
            return self.client.sessions.remove_job(session_id, job_id)
        except Exception:
            pass

    def add_job(self, session_id, job_id):
        return self.client.sessions.add_job(session_id, job_id)

    def jobs(self, session_id):
        session = self.get(session_id, json=True)
        jobs = []

        if session is None:
            return jobs

        try:
            jobs = [utils.JobsInSessionObject(k,
                                              session_id,
                                              v['client_id'],
                                              v['result'])
                    for k, v in session['jobs'].iteritems()]
        except AttributeError as error:
            LOG.error(error.message)
        return jobs

    def _build(self, session):
        session = utils.create_dict(**session)
        scheduling = {}
        utils.assign_and_remove(session, scheduling, 'schedule_start_date')
        utils.assign_and_remove(session, scheduling, 'schedule_interval')
        utils.assign_and_remove(session, scheduling, 'schedule_end_date')
        session['jobs'] = {}
        session['schedule'] = scheduling
        return self.client.sessions.create(session)


class Action(object):

    def __init__(self, request):
        self.request = request
        self.client = client(request)

    def list(self, json=False, limit=100, offset=0, search=None):
        if search:
            search = {"match": [{"_all": search}, ], }

        actions = self.client.actions.list(limit=limit,
                                           offset=offset,
                                           search=search)

        if json:
            return actions

        return [utils.ActionObjectDetail(
            action.get('action_id'),
            action['freezer_action'].get('action'),
            action['freezer_action'].get('backup_name'),
            action['freezer_action'].get('path_to_backup')
            or action['freezer_action'].get('restore_abs_path'),
            action['freezer_action'].get('storage'),
            mode=action['freezer_action'].get('mode')
        ) for action in actions]

    def get(self, job_id, json=False):

        action = self.client.actions.get(job_id)

        if json:
            return action

        return utils.ActionObjectDetail(
            action.get('action_id'),
            action['freezer_action'].get('action'),
            action['freezer_action'].get('backup_name'),
            action['freezer_action'].get('path_to_backup'),
            action['freezer_action'].get('storage'))

    def create(self, action):
        return self._build(action)

    def update(self, action, action_id):
        updated_action = {}
        updated_action['freezer_action'] = utils.create_dict(**action)
        try:
            if action['mandatory'] != '':
                updated_action['mandatory'] = action['mandatory']
        except KeyError:
            pass

        try:
            if action['max_retries'] != '':
                updated_action['max_retries'] = action['max_retries']
        except KeyError:
            pass

        try:
            if action['max_retries_interval'] != '':
                updated_action['max_retries_interval'] =\
                    action['max_retries_interval']
        except KeyError:
            pass
        return self.client.actions.update(action_id, updated_action)

    def delete(self, action_id):
        return self.client.actions.delete(action_id)

    def _build(self, action):
        """Get a flat action dict and convert it to a freezer action format
        """
        action_rules = {}

        utils.assign_and_remove(action, action_rules, 'max_retries')
        utils.assign_and_remove(action, action_rules, 'max_retries_interval')
        utils.assign_and_remove(action, action_rules, 'mandatory')
        action = utils.create_dict(**action)
        action.pop('action_id', None)
        # if the backup name has spaces the tar metadata file cannot be found
        # so we replace " " for "_"
        backup_name = action.pop('backup_name', None)
        action['backup_name'] = backup_name.replace(' ', '_')
        action = {'freezer_action': action}
        action.update(action_rules)
        return self.client.actions.create(action)


class Client(object):

    def __init__(self, request):
        self.request = request
        self.client = client(request)

    def list(self, json=False, limit=100, offset=0, search=None):
        if search:
            search = {"match": [{"_all": search}, ], }

        clients = self.client.registration.list(limit=limit,
                                                offset=offset,
                                                search=search)

        if json:
            return clients

        return [utils.ClientObject(
            c.get('client', {}).get('hostname'),
            c.get('client', {}).get('client_id'),
            c.get('uuid')
        ) for c in clients]

    def get(self, client_id, json=False):
        c = self.client.registration.get(client_id)

        if json:
            return c

        return utils.ClientObject(
            c.get('client', {}).get('hostname'),
            c.get('client', {}).get('client_id'),
            c.get('uuid'))

    def delete(self, client_id):
        return self.client.registration.delete(client_id)


class Backup(object):

    def __init__(self, request):
        self.request = request
        self.client = client(request)

    def list(self, json=False, limit=30, offset=0, search=None):
        if search:
            search = {"match": [{"_all": search}, ], }

        backups = self.client.backups.list(limit=limit,
                                           offset=offset,
                                           search=search)

        if json:
            return backups

        return [utils.BackupObject(
            backup_id=b.get('backup_uuid'),
            action=b.get('backup_metadata', {}).get('action'),
            time_stamp=b.get('backup_metadata', {}).get('time_stamp'),
            backup_name=b.get('backup_metadata', {}).get('backup_name'),
            backup_media=b.get('backup_metadata', {}).get('backup_media'),
            path_to_backup=b.get('backup_metadata', {}).get('path_to_backup'),
            hostname=b.get('backup_metadata', {}).get('hostname'),
            container=b.get('backup_metadata', {}).get('container'),
            level=b.get('backup_metadata', {}).get('level'),
            curr_backup_level=b.get('backup_metadata', {}).get(
                'curr_backup_level'),
            encrypted=b.get('backup_metadata', {}).get('encrypted'),
            total_broken_links=b.get('backup_metadata', {}).get(
                'total_broken_links'),
            excluded_files=b.get('backup_metadata', {}).get('excluded_files'),
            storage=b.get('backup_metadata', {}).get('storage'),
            ssh_host=b.get('backup_metadata', {}).get('ssh_host'),
            ssh_key=b.get('backup_metadata', {}).get('ssh_key'),
            ssh_username=b.get('backup_metadata', {}).get('ssh_username'),
            ssh_port=b.get('backup_metadata', {}).get('ssh_port'),
            mode=b.get('backup_metadata', {}).get('ssh_mode'),
        ) for b in backups]

    def get(self, backup_id, json=False):

        search = {"match": [{"backup_uuid": backup_id}, ], }

        b = self.client.backups.list(limit=1, search=search)
        b = b[0]

        if json:
            return b

        return utils.BackupObject(
            backup_id=b.get('backup_uuid'),
            action=b.get('backup_metadata', {}).get('action'),
            time_stamp=b.get('backup_metadata', {}).get('time_stamp'),
            backup_name=b.get('backup_metadata', {}).get('backup_name'),
            backup_media=b.get('backup_metadata', {}).get('backup_media'),
            path_to_backup=b.get('backup_metadata', {}).get('path_to_backup'),
            hostname=b.get('backup_metadata', {}).get('hostname'),
            container=b.get('backup_metadata', {}).get('container'),
            level=b.get('backup_metadata', {}).get('level'),
            curr_backup_level=b.get('backup_metadata', {}).get(
                'curr_backup_level'),
            encrypted=b.get('backup_metadata', {}).get('encrypted'),
            total_broken_links=b.get('backup_metadata', {}).get(
                'total_broken_links'),
            excluded_files=b.get('backup_metadata', {}).get('excluded_files'),
            storage=b.get('backup_metadata', {}).get('storage'),
            ssh_host=b.get('backup_metadata', {}).get('ssh_host'),
            ssh_key=b.get('backup_metadata', {}).get('ssh_key'),
            ssh_username=b.get('backup_metadata', {}).get('ssh_username'),
            ssh_port=b.get('backup_metadata', {}).get('ssh_port'),
            mode=b.get('backup_metadata', {}).get('ssh_mode'),
        )

    def restore(self, data):
        backup = self.get(data['backup_id'])
        client_id = data['client']
        name = "Restore {0} for {1}".format(backup.backup_name, client_id)

        iso_date = utils.timestamp_to_iso(backup.time_stamp)

        action = {
            'action': 'restore',
            'backup_name': backup.backup_name,
            'restore_abs_path': data['path'],
            'container': backup.container,
            'restore_from_host': backup.hostname,
            'storage': backup.storage,
            'restore_from_date': iso_date
        }

        if backup.storage == 'ssh':
            action['ssh_host'] = backup.ssh_host
            action['ssh_key'] = backup.ssh_key
            action['ssh_username'] = backup.ssh_username
            action['ssh_port'] = backup.ssh_port

        action_id = Action(self.request).create(action)

        job = {
            'job_actions': [{
                'action_id': action_id,
                'freezer_action': action
            }],
            'client_id': client_id,
            'description': name,
            'job_schedule': {}
        }
        job_id = self.client.jobs.create(job)
        return Job(self.request).start(job_id)



# (c) Copyright 2014,2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.utils.translation import ugettext_lazy as _

import horizon


class FreezerDR(horizon.PanelGroup):
    slug = "freezerdr"
    name = _("Backup and Restore")
    panels = ('jobs', 'actions', 'sessions', 'clients', 'backups')


class Freezer(horizon.Dashboard):
    name = _("Disaster Recovery")
    slug = "disaster_recovery"
    panels = (FreezerDR,)
    default_panel = 'jobs'


horizon.register(Freezer)



# (c) Copyright 2014,2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
from mock import patch

from horizon.utils.urlresolvers import reverse
import openstack_dashboard.test.helpers as helpers


@patch('freezer.apiclient.client')
class TestRestApi(helpers.TestCase):
    CLIENT_1 = {u'client': {u'hostname': u'jonas',
                            u'description': u'client description',
                            u'client_id': u'test-client',
                            u'config_ids': [u'fdaf2fwf2', u'fdsfdsfdsfs']},
                u'user_id': u'13c2b15308c04cdf86989ee7335eb504'}

    JSON_PREFIX = ')]}\',\n'

    def test_clients_get(self, client_mock):
        client_mock.Client().registration.list.return_value = [self.CLIENT_1]
        url = reverse("horizon:freezer_ui:api_clients")

        res = self.client.get(url, HTTP_X_REQUESTED_WITH='XMLHttpRequest')

        self.assertEqual(200, res.status_code)
        self.assertEqual('application/json', res['content-type'])
        self.assertEqual(self.JSON_PREFIX + json.dumps([self.CLIENT_1]),
                         res.content)
        # there is no get ALL api at the moment, so we just fetch a big number
        client_mock.Client().registration.list.assert_called_once_with(
            limit=9999)



# (c) Copyright 2014,2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

from django import shortcuts
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import workflows

import disaster_recovery.api.api as freezer_api


LOG = logging.getLogger(__name__)


class ActionsConfigurationAction(workflows.Action):
    actions = forms.CharField(
        required=False)

    job_id = forms.CharField(
        required=False)

    class Meta(object):
        name = _("Actions")
        slug = "actions"
        help_text_template = "disaster_recovery/jobs" \
                             "/_actions.html"


class ActionsConfiguration(workflows.Step):
    action_class = ActionsConfigurationAction
    contributes = ('actions', 'job_id')


class UpdateActions(workflows.Workflow):
    slug = "update_actions"
    name = _("Update Actions")
    finalize_button_name = _("Save")
    success_message = _('Actions updated correctly.')
    failure_message = _('Unable to update actions.')
    success_url = "horizon:disaster_recovery:jobs:index"
    default_steps = (ActionsConfiguration,)

    def handle(self, request, context):
        try:
            if context['job_id'] != '':
                freezer_api.Job(request).update(context['job_id'], context)
            return shortcuts.redirect('horizon:disaster_recovery:jobs:index')
        except Exception:
            exceptions.handle(request)
            return False



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

import logging

from django import shortcuts
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import workflows

import disaster_recovery.api.api as freezer_api


LOG = logging.getLogger(__name__)


class ActionsConfigurationAction(workflows.Action):
    pass

    class Meta(object):
        name = _("Actions")
        slug = "actions"
        help_text_template = "disaster_recovery/jobs" \
                             "/_actions.html"


class ActionsConfiguration(workflows.Step):
    action_class = ActionsConfigurationAction
    contributes = ()


class InfoAction(workflows.Action):
    job_id = forms.CharField(label=_("Job ID"), required=False,
                             widget=forms.HiddenInput(),)
    actions = forms.CharField(label=_("Actions"), required=False,
                              widget=forms.TextInput(
                                  attrs={'readonly': 'readonly'}))

    def __init__(self, request, *args, **kwargs):
        super(InfoAction, self).__init__(request, *args, **kwargs)

    class Meta(object):
        name = _("Info")
        # Unusable permission so this is always hidden. However, we
        # keep this step in the workflow for validation/verification purposes.
        permissions = ()


class Info(workflows.Step):
    action_class = InfoAction
    contributes = ("job_id", "actions")


class ConfigureActions(workflows.Workflow):
    slug = "job"
    name = _("Actions Configuration")
    finalize_button_name = _("Save")
    success_message = _('Actions saved correctly.')
    failure_message = _('Unable to save actions.')
    success_url = "horizon:disaster_recovery:jobs:index"
    default_steps = (ActionsConfiguration, Info)

    def handle(self, request, context):
        try:
            if context['job_id'] != '':
                freezer_api.Job(request).update_actions(context['job_id'],
                                                        context['actions'])
            return shortcuts.redirect('horizon:disaster_recovery:jobs:index')
        except Exception:
            exceptions.handle(request)
            return False



# (c) Copyright 2014,2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import functools
import json

from django.http import HttpResponse
from django.views import generic

from openstack_dashboard.api.rest import utils as rest_utils
from openstack_dashboard.api.rest.utils import JSONResponse

import disaster_recovery.api.api as freezer_api


# https://github.com/tornadoweb/tornado/issues/1009
# http://haacked.com/archive/2008/11/20/anatomy-of-a-subtle-json-vulnerability.aspx/
def prevent_json_hijacking(function):
    @functools.wraps(function)
    def wrapper(*args, **kwargs):
        response = function(*args, **kwargs)
        if isinstance(response, JSONResponse) and response.content:
            response.content = ")]}',\n" + response.content
        return response

    return wrapper


class Clients(generic.View):
    """API for nova limits."""

    @prevent_json_hijacking
    @rest_utils.ajax()
    def get(self, request, job_id=None):
        """Get all registered freezer clients"""

        # we don't have a "get all clients" api (probably for good reason) so
        # we need to resort to getting a very high number.
        clients = freezer_api.Client(request).list(json=True)
        clients = json.dumps(clients)
        return HttpResponse(clients, content_type="application/json")


class ActionList(generic.View):
    @prevent_json_hijacking
    @rest_utils.ajax()
    def get(self, request):
        """Get all registered freezer actions"""

        actions = freezer_api.Action(request).list(json=True)
        actions = json.dumps(actions)
        return HttpResponse(actions, content_type="application/json")


class Actions(generic.View):
    @prevent_json_hijacking
    @rest_utils.ajax()
    def get(self, request, job_id=None):
        actions = freezer_api.Action(request).list(json=True)
        actions_in_job = freezer_api.Job(request).actions(job_id, api=True)

        action_ids = [a['action_id'] for a in actions]
        actions_in_job_ids = [a['action_id'] for a in actions_in_job]

        available = set.difference(set(action_ids), set(actions_in_job_ids))
        selected = set.intersection(set(action_ids), set(actions_in_job_ids))

        available_actions = []
        for action in actions:
            if action['action_id'] in available:
                available_actions.append(action)

        selected_actions = []
        for action in actions_in_job:
            if action['action_id'] in selected:
                selected_actions.append(action)

        actions = {'available': available_actions,
                   'selected': selected_actions}

        actions = json.dumps(actions)
        return HttpResponse(actions, content_type="application/json")
